@@ -96,26 +96,6 @@ static TProfile *Current = 0;
  */
 static TProfile *Base = 0;
 
-/*
- * a set handler is a function which is called every time a gnome_config_set_*
- * function is called, this can be used by the app to say guarantee a sync,
- * apps using libgnomeui should not call this, libgnomeui already provides
- * this, this would be for non-gui apps and apps that use a different toolkit
- * than gtk.
- */
-static void (*set_handler)(void *data) = NULL;
-static void *set_handler_data;
-
-#define CALL_SET_HANDLER() { if(set_handler) (*set_handler)(set_handler_data); }
-
-/* same as above for a "sync" handler */
-static void (*sync_handler)(void *data) = NULL;
-static void *sync_handler_data;
-
-#define CALL_SYNC_HANDLER() { if(sync_handler) (*sync_handler)(sync_handler_data); }
-
-
-
 /* The `release_path' and `parsed_path' routines are inside the
    following file.  It is in a separate file to allow the test-suite
    to get at it, without needing to export special symbols.
@@ -565,13 +545,14 @@ free_profile (TProfile *p)
 
 
 static void 
-dump_profile (TProfile *p)
+dump_profile (TProfile *p, int one_only)
 {
 	FILE *profile;
     
 	if (!p)
 		return;
-	dump_profile (p->link);
+	if(!one_only)
+		dump_profile (p->link,FALSE);
 	
 	/*
 	 * was this profile written to?, if not it's not necessary to dump
@@ -625,8 +606,56 @@ dump_profile (TProfile *p)
 void 
 gnome_config_sync (void)
 {
-	dump_profile (Base);
-	CALL_SYNC_HANDLER();
+	dump_profile (Base,FALSE);
+	gnome_config_drop_all();
+}
+
+/**
+ * gnome_config_sync_file:
+ * @path: A gnome-config path
+ *
+ * Writes all of the information modified by gnome-config to the
+ * disk for the given file.
+ *
+ * Note: the gnome_config code does not write anything to the
+ * configuration files until this routine or gnome_config_sync 
+ * is actually invoked.
+ */
+/**
+ * gnome_config_private_sync_file:
+ * @path: A gnome-config path
+ *
+ * Writes all of the information modified by gnome-config to the
+ * disk for the given private file.
+ *
+ * Note: the gnome_config code does not write anything to the
+ * configuration files until this routine or gnome_config_sync 
+ * is actually invoked.
+ */
+void 
+_gnome_config_sync_file (char *path, int priv)
+{
+	TProfile *p;
+	ParsedPath *pp;
+	char *fake_path;
+	
+	if (!path)
+		return;
+
+	fake_path = g_concat_dir_and_file (path, "section/key");
+	pp = parse_path (fake_path, priv);
+	g_free (fake_path);
+
+	for (p = Base; p; p = p->link){
+		if (strcmp (pp->file, p->filename) != 0)
+			continue;
+		if(!p->written_to)
+			break;
+		dump_profile (p,TRUE);
+		gnome_config_drop_file(path);
+		break;
+	}
+	release_path (pp);
 }
 
 /**
@@ -1329,7 +1358,6 @@ _gnome_config_set_translated_string (const char *path, const char *value,
 		g_free (tkey);
 	} else
 		_gnome_config_set_string (path, value, priv);
-	CALL_SET_HANDLER();
 }
 
 /**
@@ -1350,7 +1378,6 @@ _gnome_config_set_string (const char *path, const char *new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key, new_value, pp->file,
 			   NULL);
 	release_path (pp);
-	CALL_SET_HANDLER();
 }
 
 /**
@@ -1373,7 +1400,6 @@ _gnome_config_set_int (const char *path, int new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key, intbuf, pp->file,
 			   NULL);
 	release_path (pp);
-	CALL_SET_HANDLER();
 }
 
 /**
@@ -1396,7 +1422,6 @@ _gnome_config_set_float (const char *path, gdouble new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key, floatbuf, pp->file,
 			   NULL);
 	release_path (pp);
-	CALL_SET_HANDLER();
 }
 
 /**
@@ -1417,7 +1442,6 @@ _gnome_config_set_bool (const char *path, gboolean new_value, gint priv)
 	r = access_config (SET, pp->section, pp->key,
 			   new_value ? "true" : "false", pp->file, NULL);
 	release_path (pp);
-	CALL_SET_HANDLER();
 }
 
 /**
@@ -1472,7 +1496,6 @@ _gnome_config_set_vector (const char *path, int argc,
 	access_config (SET, pp->section, pp->key, s, pp->file, NULL);
 	g_free (s);
 	release_path (pp);
-	CALL_SET_HANDLER();
 }
 
 /**
@@ -1513,15 +1536,13 @@ gnome_config_pop_prefix (void)
 void
 gnome_config_set_set_handler(void (*func)(void *),void *data)
 {
-	set_handler = func;
-	set_handler_data = data;
+	g_warning("gnome_config_set_set_handler is obscolete and has no replacement");
 }
 
 void
 gnome_config_set_sync_handler(void (*func)(void *),void *data)
 {
-	sync_handler = func;
-	sync_handler_data = data;
+	g_warning("gnome_config_set_sync_handler is obscolete and has no replacement");
 }
 
 #ifdef TEST
