@@ -35,28 +35,27 @@ extern int errno;
 #endif
 
 /**
- * gnome_execute_async_with_env:
+ * gnome_execute_async_with_env_fds:
  * @dir: Directory in which child should be execd, or NULL for current
  *       directory
  * @argc: Number of arguments
  * @argv: Argument vector to exec child
  * @envc: Number of environment slots
  * @envv: Environment vector
+ * @close_fds: If TRUE will close all fds but 0,1, and 2
  * 
- * This function forks and executes some program in the background.
- * On error, returns %-1; in this case, errno should hold a useful
- * value.  Searches the path to find the child.  Environment settings
- * in @envv are added to the existing environment -- they do not
- * completely replace it.
+ * Like gnome_execute_async_with_env() but has a flag to decide whether or not  * to close fd's
  * 
  * Return value: the process id, or %-1 on error.
  **/
 int
-gnome_execute_async_with_env (const char *dir, int argc, char * const argv[],
-			      int envc, char * const envv[])
+gnome_execute_async_with_env_fds (const char *dir, int argc, 
+				       char * const argv[], int envc, 
+				       char * const envv[], 
+				       gboolean close_fds)
 {
   int comm_pipes[2];
-  int child_errno, itmp;
+  int child_errno, itmp, i, open_max;
   char **cpargv;
   pid_t child_pid, immediate_child_pid; /* XXX this routine assumes
 					   pid_t is signed */
@@ -103,6 +102,13 @@ gnome_execute_async_with_env (const char *dir, int argc, char * const argv[],
       memcpy(cpargv, argv, argc * sizeof(char *));
       cpargv[argc] = NULL;
 
+      if(close_fds) {
+	/* Close all file descriptors but stdin stdout and stderr */
+	open_max = sysconf (_SC_OPEN_MAX);
+	for (i = 3; i < open_max; i++){
+	  close (i);
+	}
+      }
       /* doit */
       execvp(cpargv[0], cpargv);
 
@@ -131,6 +137,32 @@ gnome_execute_async_with_env (const char *dir, int argc, char * const argv[],
   return child_pid;
 }
 
+/**
+ * gnome_execute_async_with_env:
+ * @dir: Directory in which child should be execd, or NULL for current
+ *       directory
+ * @argc: Number of arguments
+ * @argv: Argument vector to exec child
+ * @envc: Number of environment slots
+ * @envv: Environment vector
+ * 
+ * This function forks and executes some program in the background.
+ * On error, returns %-1; in this case, errno should hold a useful
+ * value.  Searches the path to find the child.  Environment settings
+ * in @envv are added to the existing environment -- they do not
+ * completely replace it.  This function closes all fds besides 0, 1,
+ * and 2 for the child
+ * 
+ * Return value: the process id, or %-1 on error.
+ **/
+int
+gnome_execute_async_with_env (const char *dir, int argc, char * const argv[], 
+			      int envc, char * const envv[])
+{
+  return gnome_execute_async_with_env_fds(dir,argc,argv,envc,envv,TRUE);
+}
+
+
 /**
  * gnome_execute_async:
  * @dir: Directory in which child should be execd, or NULL for current
@@ -149,6 +181,58 @@ gnome_execute_async (const char *dir, int argc, char * const argv[])
   return gnome_execute_async_with_env (dir, argc, argv, 0, NULL);
 }
 
+/**
+ * gnome_execute_async_fds:
+ * @dir: Directory in which child should be execd, or NULL for current
+ *       directory
+ * @argc: Number of arguments
+ * @argv: Argument vector to exec child
+ * @close_fds: 
+ * Like gnome_execute_async_with_env_fds(), but doesn't add anything to
+ * child's environment.
+ * 
+ * Return value: process id of child, or %-1 on error.
+ **/
+int
+gnome_execute_async_fds (const char *dir, int argc, 
+				 char * const argv[], gboolean close_fds)
+{
+  return gnome_execute_async_with_env_fds (dir, argc, argv, 0, NULL, 
+					   close_fds);
+}
+
+/**
+ * gnome_execute_shell_fds:
+ * @dir: Directory in which child should be execd, or NULL for current
+ *       directory
+ * @commandline: Shell command to execute
+ * @close_fds: Like close_fds in gnome_execute_async_with_env_fds()
+ * Like gnome_execute_async_with_env_fds(), but uses the user's shell to
+ * run the desired program.  Note that the pid of the shell is
+ * returned, not the pid of the user's program.
+ * 
+ * Return value: process id of shell, or %-1 on error.
+ **/
+int
+gnome_execute_shell_fds (const char *dir, const char *commandline,
+			      gboolean close_fds)
+{
+  char * argv[4];
+  int r;
+
+  g_return_val_if_fail(commandline != NULL, -1);
+
+  argv[0] = gnome_util_user_shell ();
+  argv[1] = "-c";
+  /* This cast is safe.  It sucks that we need it, though.  */
+  argv[2] = (char *) commandline;
+  argv[3] = NULL;
+
+  r = gnome_execute_async_with_env_fds (dir, 4, argv, 0, NULL, close_fds);
+  free (argv[0]);
+  return r;
+}
+
 /**
  * gnome_execute_shell:
  * @dir: Directory in which child should be execd, or NULL for current
@@ -164,18 +248,12 @@ gnome_execute_async (const char *dir, int argc, char * const argv[])
 int
 gnome_execute_shell (const char *dir, const char *commandline)
 {
-  char * argv[4];
-  int r;
-
-  g_return_val_if_fail(commandline != NULL, -1);
-
-  argv[0] = gnome_util_user_shell ();
-  argv[1] = "-c";
-  /* This cast is safe.  It sucks that we need it, though.  */
-  argv[2] = (char *) commandline;
-  argv[3] = NULL;
-
-  r = gnome_execute_async (dir, 4, argv);
-  free (argv[0]);
-  return r;
+  return gnome_execute_shell_fds(dir, commandline, TRUE);
 }
+
+
+
+
+
+
+
