@@ -111,7 +111,6 @@ static char *lock_directory;
 #define LIST "list"
 #define LISTLEN 4
 
-
 
 
 /* Initialize the database.  Returns 0 on success.  */
@@ -141,6 +140,7 @@ lock (void)
 		/* We use a lock directory and not flock/fcntl because
 		   we want this to work when the database is on an NFS
 		   filesystem.  Sigh.  */
+
 		while (mkdir (lock_directory, 0)) {
 			if (errno != EEXIST) {
 				/* Don't know what to do here.  */
@@ -165,6 +165,32 @@ unlock (void)
 	}
 }
 
+/**
+ * gnome_metadata_lock:
+ *
+ * Locks the metadata system.  Used if you are going to invoke
+ * many metadata operations to speed up metadata access.
+ */
+void
+gnome_metadata_lock ()
+{
+	if (! database)
+		init ();
+	lock ();
+}
+
+/**
+ * gnome_metadata_lock:
+ *
+ * Unlocks the metadata system.  Used if you are going to invoke
+ * many metadata operations to speed up metadata access.
+ */
+void
+gnome_metadata_unlock ()
+{
+	unlock ();
+}
+
 /* Set a piece of metadata.  */
 static GnomeMetadataError_t
 metadata_set (const char *space, const char *object, const char *key,
@@ -355,6 +381,9 @@ metadata_get_list (const char *space, const char *object, DBT *value)
 	int key_size = sl + ol + LISTLEN, r;
 	DBT key;
 
+	if (!database && init ())
+		return GNOME_METADATA_IO_ERROR;
+	
 	dbkey = alloca (key_size);
 	strcpy (dbkey, space);
 	strcat (dbkey, LIST);
