@@ -16,11 +16,14 @@ struct _GnomeTrigger {
 	   gnome_triggers_do got */
 	  GnomeTriggerActionFunction function;
 	  gchar *command;
-	  gchar *mediafile;
+	  struct {
+	    gchar *file;
+	    int cache_id;
+	  } media;
 	} u;
         gchar *level;
 };
-typedef struct _GnomeTrigger * GnomeTrigger;
+typedef struct _GnomeTrigger GnomeTrigger;
 
 /* Must be called before doing any triggers stuff */
 void gnome_triggers_init(void);
@@ -34,13 +37,14 @@ gint gnome_triggers_readfile(gchar *filename);
 		     "warning", "system", "device", "disk", "/dev/hda1");
 */
 
-void gnome_triggers_add_trigger(GnomeTrigger nt, ...);
-void gnome_triggers_vadd_trigger(GnomeTrigger nt,
+void gnome_triggers_add_trigger(GnomeTrigger *nt, ...);
+void gnome_triggers_vadd_trigger(GnomeTrigger *nt,
 				 char *supinfo[]);
 
-void gnome_triggers_do(char *msg, char *level, ...);
+void gnome_triggers_do(const char *msg, const char *level, ...);
 
-void gnome_triggers_vdo(char *msg, char *level, char *supinfo[]);
+void gnome_triggers_vdo(const char *msg, const char *level,
+			const char *supinfo[]);
 
 void gnome_triggers_destroy(void);
 
