@@ -1,7 +1,7 @@
 /*
  *
  * Gnome utility routines.
- * (C)  1997, 1998 the Free Software Foundation.
+ * (C)  1997, 1998, 1999 the Free Software Foundation.
  *
  * Author: Miguel de Icaza, 
  */
@@ -269,7 +269,6 @@ g_file_exists (const char *filename)
 	return stat (filename, &s) == 0;
 }
 
-
 /**
  * g_copy_strings:
  * @first: first string
@@ -308,7 +307,6 @@ g_copy_strings (const char *first, ...)
 	
 	return result;
 }
-#define g_copy_strings g_strconcat
 
 /**
  * g_unix_error_string:
@@ -516,3 +514,35 @@ g_is_image_filename (const char * path)
 
 
 
+/**
+ * gnome_is_program_in_path:
+ * @program: a program name.
+ *
+ * Looks for program in the PATH, if it is found, a g_strduped
+ * string with the full path name is returned.
+ *
+ * Returns NULL if program is not on the path or a string 
+ * allocated with g_malloc with the full path name of the program
+ * found
+ */
+gchar *
+gnome_is_program_in_path (const gchar *program)
+{
+	static gchar **paths = NULL;
+	gchar **p;
+	gchar *f;
+	
+	if (!paths)
+	  paths = g_strsplit(getenv("PATH"), ":", -1);
+
+	p = paths;
+	while (*p){
+		f = g_strconcat (*p,"/",program, NULL);
+		if (g_file_exists (f))
+			return f;
+		g_free (f);
+		p++;
+	}
+	return 0;
+}
+
