@@ -442,8 +442,7 @@ gnome_fill_info (GtkWidget *widget,
 
 	/* fill struct */
 	if(title) 
-	  gai->title = g_copy_strings (title, " ",
-				       version ? version : "", NULL);
+	  gai->title = g_strconcat (title, " ", version ? version : "", NULL);
 	else
 	  gai->title = NULL;
   
@@ -586,6 +585,8 @@ gnome_about_construct (GnomeAbout *about,
 			      title, version, 
 			      copyright, authors, 
 			      comments, logo);
+	gtk_signal_connect(GTK_OBJECT(about),"destroy",
+			   GTK_SIGNAL_FUNC(gnome_destroy_about),ai);
   
 	w = ai->w; h = ai->h;
 	/* x = (gdk_screen_width ()  - w) / 2; */
