@@ -44,7 +44,7 @@ typedef enum {
 	GNOME_MDI_NOTEBOOK,
 	GNOME_MDI_TOPLEVEL,
 	GNOME_MDI_MODAL,
-	GNOME_MDI_DEFAULT_MODE = 42,
+	GNOME_MDI_DEFAULT_MODE = 42
 } GnomeMDIMode;
 
 struct _GnomeMDI {
