@@ -39,6 +39,7 @@ static void gnome_proc_bar_setup_colors (GnomeProcBar *pb);
 static void gnome_proc_bar_draw (GnomeProcBar *pb, unsigned val []);
 static void gnome_proc_bar_destroy (GtkObject *obj);
 
+
 guint
 gnome_proc_bar_get_type (void)
 {
@@ -309,7 +310,7 @@ gnome_proc_bar_set_values (GnomeProcBar *pb, unsigned val [])
 }
 
 void
-gnome_proc_bar_start (GnomeProcBar *pb, gint time, gpointer data)
+gnome_proc_bar_start (GnomeProcBar *pb, gint gtime, gpointer data)
 
 {
     g_return_if_fail (pb != NULL);
@@ -320,7 +321,7 @@ gnome_proc_bar_start (GnomeProcBar *pb, gint time, gpointer data)
 
     if (pb->cb) {
 	pb->cb (data);
-	pb->tag = gtk_timeout_add (time, pb->cb, data);
+	pb->tag = gtk_timeout_add (gtime, pb->cb, data);
     }
 }
 
