@@ -1,7 +1,4 @@
 /* needed for sigaction under 'gcc -ansi -pedantic' on GNU/Linux */
-#ifndef _POSIX_SOURCE
-#define _POSIX_SOURCE 1
-#endif
 #include <sys/types.h>
 
 #include <unistd.h>
@@ -166,7 +163,7 @@ name_server_by_forking (CORBA_Environment *ev)
 		if (iorbuf[strlen(iorbuf)-1] == '\n')
 			iorbuf[strlen(iorbuf)-1] = '\0';
 		
-		name_service = CORBA_ORB_string_to_object(gnome_orbit_orb, iorbuf, &ev);
+		name_service = CORBA_ORB_string_to_object((CORBA_ORB)gnome_orbit_orb, iorbuf, ev);
 	} else if (fork ()) {
 		/* de-zombifier process, just exit */
 		_exit(0);
