@@ -5,7 +5,12 @@
 # include <config.h>
 #endif
 
+#include <stddef.h>
+/* ??? This is required to get `size_t' on some systems.  */
+#include <sys/types.h>
+
 #include <gnomesupport.h>
+
 #include <gnome-argp.h>
 
 /* Override some of config.h.
