@@ -874,7 +874,8 @@ static char *vt_expand_line(struct vt_line *l, int start, int end, char *out)
   Line '0' is the top of the screen, negative lines are part of the
   scrollback, positive lines are part of the visible screen.
 */
-char *vt_select_block(struct _vtx *vx, int sx, int sy, int ex, int ey, int *len)
+static char *
+vt_select_block(struct _vtx *vx, int sx, int sy, int ex, int ey, int *len)
 {
   struct vt_line *wn, *nn;
   int line;
