@@ -42,9 +42,6 @@
 #include "subshell.h"
 
 
-#define DOUBLE_BUFFER		/* use back buffer to represent screen
-				   FIXME: THIS MUST BE DEFINED */
-
 /* define to 'x' to enable copius debug of this module */
 #define d(x)
 
@@ -62,8 +59,12 @@ void vt_delete_lines(struct vt_em *vt, int count);
 void vt_clear_lines(struct vt_em *vt, int top, int count);
 void vt_clear_line_portion(struct vt_em *vt, int start_col, int end_col);
 
+static void vt_resize_lines(struct vt_line *wm, int width, uint32 default_attr);
+
+
 static unsigned char vt_remap_dec[256];
 
+
 #ifdef DEBUG
 static void
 dump_scrollback(struct vt_em *vt)
@@ -209,7 +210,7 @@ vt_scrollback_add(struct vt_em *vt, struct vt_line *wn)
   struct vt_line *ln;
 
   /* create a new scroll-back line */
-  ln = g_malloc(sizeof(struct vt_line) + (sizeof(uint32) * wn->width));
+  ln = g_malloc(VT_LINE_SIZE(wn->width));
   ln->next = NULL;
   ln->prev = NULL;
   ln->width = wn->width;
@@ -221,25 +222,31 @@ vt_scrollback_add(struct vt_em *vt, struct vt_line *wn)
   ln->line = ln->prev->line + 1;
   
   /* limit the total number of lines in scrollback */
-  if (vt->scrollbacklines >= vt->scrollbackmax) {
-    /* remove the top of list line */
-    ln = (struct vt_line *)vt_list_remhead(&vt->scrollback);
-    g_free(ln);
+  if (vt->scrollbacklines >= vt->scrollbackmax) 
+    {
+      /* remove the top of list line */
+      ln = (struct vt_line *)vt_list_remhead(&vt->scrollback);
+      g_free(ln);
 
-    /* need to track changes to this, even if they're not 'real' */
-    if (vt->scrollbackoffset) {
-      vt->scrollbackold--;
-      if ((-vt->scrollbackoffset) < vt->scrollbackmax)
-	vt->scrollbackoffset--;
+      /* need to track changes to this, even if they're not 'real' */
+      if (vt->scrollbackoffset)
+	{
+	  vt->scrollbackold--;
+	  if ((-vt->scrollbackoffset) < vt->scrollbackmax)
+	    vt->scrollbackoffset--;
+	}
+    } 
+  else 
+    {
+      vt->scrollbacklines++;
+
+      /* we've effectively moved the 'old' scrollback position */
+      if (vt->scrollbackoffset)
+	{
+	  vt->scrollbackold--;
+	  vt->scrollbackoffset--;
+	}
     }
-  } else {
-    vt->scrollbacklines++;
-    /* we've effectively moved the 'old' scrollback position */
-    if (vt->scrollbackoffset) {
-      vt->scrollbackold--;
-      vt->scrollbackoffset--;
-    }
-  }
 }
 
 
@@ -258,7 +265,8 @@ vt_scroll_up(struct vt_em *vt, int count)
   int i;
   int blank;
 
-  d(printf("vt_scroll_up(%d) (top = %d bottom=%d)\n", count, vt->scrolltop, vt->scrollbottom));
+  d(printf("vt_scroll_up count=%d top=%d bottom=%d\n", 
+	   count, vt->scrolltop, vt->scrollbottom));
 
   blank = vt->attr;
 
@@ -311,7 +319,9 @@ vt_scroll_down(struct vt_em *vt, int count)
   struct vt_line *wn, *nn;
   int i;
 
-  d(printf("vt_scroll_down(%d) (top = %d bottom=%d)\n", count, vt->scrolltop, vt->scrollbottom));
+  d(printf("vt_scroll_down count=%d top=%d bottom=%d\n",
+	   count, vt->scrolltop, vt->scrollbottom));
+
   /* FIXME: do this properly */
 
   while(count>0) {
@@ -392,7 +402,8 @@ void vt_insert_lines(struct vt_em *vt, int count)
   struct vt_line *wn, *nn;
   int i;
 
-  d(printf("vt_insert_lines(%d) (top = %d bottom = %d cursory = %d)\n", count, vt->scrolltop, vt->scrollbottom, vt->cursory));
+  d(printf("vt_insert_lines(%d) (top = %d bottom = %d cursory = %d)\n",
+	   count, vt->scrolltop, vt->scrollbottom, vt->cursory));
 
   /* FIXME: Do this properly */
 
@@ -533,9 +544,10 @@ static void vt_tab(struct vt_em *vt)
   d(printf("tab\n"));
 
   l = vt->this_line;
-  c=l->data[vt->cursorx]&VTATTR_DATAMASK;
+  c = l->data[vt->cursorx] & VTATTR_DATAMASK;
+
   /* dont store tab over a space - will affect attributes */
-  if (c==0) {
+  if (c == 0) {
     /* We do not store the attribute as tabs are transparent
      * with respect to attributes
      */
@@ -615,9 +627,11 @@ vt_insert_line(struct vt_em *vt)
 static void
 vt_delete_line(struct vt_em *vt)
 {
-  d(printf("delete line(s)\n"));
+  d(printf("vt_delete_line\n"));
+
   if (vt->argcnt==0) {
-    if (vt->cursory>vt->scrolltop) {	/* reverse line feed, not delete */
+    if (vt->cursory > vt->scrolltop) {	/* reverse line feed, not delete */
+      d(printf("vt_delete_line: should we try to scroll up?\n"));
       vt->cursory--;
     } else {
       vt_scroll_down(vt, 1);
@@ -625,8 +639,9 @@ vt_delete_line(struct vt_em *vt)
   } else if (vt->argcnt==1) {
     vt_delete_lines(vt, atoi(vt->args[0]));/* insert multiple characters */
   } else {
-    d(printf("delete characters got >1 parameters\n"));
+    d(printf("vt_delete_line: delete characters got >1 parameters\n"));
   }
+
   vt->this_line = (struct vt_line *)vt_list_index(&vt->lines, vt->cursory);
 }
 
@@ -957,7 +972,6 @@ vt_dsr(struct vt_em *vt)
 static void
 vt_scroll(struct vt_em *vt)
 {
-  d(printf("set scroll region called\n"));
   if (vt->argcnt==0) {
     vt->scrolltop = 0;
     vt->scrollbottom = vt->height-1;
@@ -973,7 +987,8 @@ vt_scroll(struct vt_em *vt)
   if (vt->scrollbottom >= vt->height)
     vt->scrollbottom = vt->height-1;
 
-  d(printf("new region = (%d - %d)\n", vt->scrolltop, vt->scrollbottom));
+  d(printf("vt_scroll: vt->scrolltop=%d vt->scrollbottom=%d\n",
+	   vt->scrolltop, vt->scrollbottom));
 }
 
 /*
@@ -1135,11 +1150,11 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
   register int state;
   register int mode;
   struct vt_jump *modes = vtjumps;
-  char *ptrend;
+  char *ptr_end;
   void (*process)(struct vt_em *vt);	/* process function */
 
   /* states:
-   *  0: normal escape mode
+   *   0: normal escape mode
    *   1: escape mode.  switch on next character.
    *   2: '[' escape mode (keep finding numbers until a command code found)
    *   3: 'O' escape mode.  switch on next character.
@@ -1148,12 +1163,16 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
    */
 
   state = vt->state;
-  ptrend = ptr+length;
-  while (ptr<ptrend) {
-    c=(*ptr++) & 0xff;		/* convert to unsigned byte */
+  ptr_end = ptr + length;
+  while (ptr < ptr_end) {
+
+    /* convert to unsigned byte */
+    c = (*ptr++) & 0xff;
     mode = modes[c & 0x7f].modes;
     process = modes[c & 0x7f].process;
-      switch(state) {
+
+    switch (state) {
+
       case 0:
 	if (mode & VT_LIT) {
 	  /* remap character? */
@@ -1182,7 +1201,10 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	}
 	/* else ignore */
 	break;
-      case 1:			/* received 'esc', next byte */
+
+
+	/* received 'esc', next byte */
+      case 1:
 	if (mode & VT_ESC) {	/* got a \Ex sequence */
 	  vt->argcnt = 0;
 	  process(vt);
@@ -1206,6 +1228,8 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	  state = 0;		/* dont understand input */
 	}
 	break;
+
+
       case 2:
 	if (mode & VT_ARG) {
 	  if (vt->outptr<vt->outend) /* truncate excessive args */
@@ -1230,13 +1254,19 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	  state=0;		/* unexpected escape sequence - ignore */
 	}
 	break;
-      case 3:			/* \EOx */
+
+
+	/* \EOx */
+      case 3:
 	if (mode & VT_EXO) {
 	  process(vt);
 	}	/* ignore otherwise */
 	state=0;
 	break;
-      case 4:			/* \E]..;...BEL */
+
+
+	/* \E]..;...BEL */
+      case 4:
 	if (c==0x07) {
 	  /* handle output */
 	  *(vt->outptr)=0;
@@ -1250,13 +1280,17 @@ vt_parse_vt (struct vt_em *vt, char *ptr, int length)
 	    *(vt->outptr)++=c;
 	}
 	break;
-      case 5:			/* \E?x */
+
+
+	/* \E?x */
+      case 5:
 	vt->args[0][1]=c;
 	modes[vt->args[0][0]].process(vt);
 	state=0;
 	break;
       }
   }
+
   vt->state = state;
 }
 
@@ -1274,7 +1308,7 @@ struct vt_line *vt_newline(struct vt_em *vt)
   int i;
   struct vt_line *l;
 
-  l = g_malloc(sizeof(struct vt_line) + (sizeof(uint32) * vt->width));
+  l = g_malloc(VT_LINE_SIZE(vt->width));
   l->prev = NULL;
   l->next = NULL;
   l->width = vt->width;
@@ -1307,11 +1341,8 @@ vt_init(struct vt_em *vt, int width, int height)
   int i;
 
   vt_list_new(&vt->lines);
-#ifdef DOUBLE_BUFFER
   vt_list_new(&vt->lines_back);
-#endif
   vt_list_new(&vt->scrollback);
-
   vt_list_new(&vt->lines_alt);
 
   vt->width = width;
@@ -1325,11 +1356,11 @@ vt_init(struct vt_em *vt, int width, int height)
     vl = vt_newline(vt);
     vl->line = i;
     vt_list_addtail(&vt->lines, (struct vt_listnode *)vl);
-#ifdef DOUBLE_BUFFER
+
     vl = vt_newline(vt);
     vl->line = i;
     vt_list_addtail(&vt->lines_back, (struct vt_listnode *)vl);
-#endif
+
     vl = vt_newline(vt);
     vl->line = i;
     vt_list_addtail(&vt->lines_alt, (struct vt_listnode *)vl);
@@ -1524,6 +1555,67 @@ vt_destroy(struct vt_em *vt)
   /* done */
 }
 
+
+static void
+vt_resize_lines(struct vt_line *wn, int width, uint32 default_attr)
+{
+  int i;
+  uint32 c;
+  struct vt_line *nn;
+
+  nn = wn->next;  
+  while (nn) {
+    /* d(printf("wn->line=%d wn->width=%d width=%d\n", wn->line, wn->width, width)); */
+    
+    /* terminal width grew */
+    if (wn->width < width)
+      {
+	/* get the attribute of the last charactor for fill */
+	if (wn->width > 0)
+	  {
+	    c = wn->data[wn->width-1] & VTATTR_MASK;
+	  }
+	else
+	  {
+	    c = default_attr;
+	  }
+
+	/* resize the line */
+	wn = g_realloc(wn, VT_LINE_SIZE(width));
+	
+	/* re-link line into linked list */
+	wn->next->prev = wn;
+	wn->prev->next = wn;
+	
+	/* if the line got bigger, fix it up */
+	for (i = wn->width; i < width; i++) 
+	  {
+	    wn->data[i] = c;
+	    wn->modcount++;
+	    }
+	
+	wn->width = width;
+      }
+    
+    /* terminal shrunk */
+    if (wn->width > width) 
+      {
+	/* resize the line */
+	wn = g_realloc(wn, VT_LINE_SIZE(width));
+	
+	/* re-link line into linked list */
+	wn->next->prev = wn;
+	wn->prev->next = wn;
+	
+	wn->width = width;
+      }
+    
+    wn = nn;
+    nn = nn->next;
+  }
+}
+
+
 /**
  * vt_resize:
  * @vt: An initalised &vt_em.
@@ -1538,7 +1630,7 @@ vt_destroy(struct vt_em *vt)
  */
 void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixheight)
 {
-  int i, count, pass, old_width;
+  int i, count, pass, old_width, remove_bottom;
   uint32 c;
   struct vt_line *wn, *nn;
 
@@ -1556,26 +1648,57 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
   if (height < vt->height) {
 
     count = vt->height - height;
+    remove_bottom = 0;
+
     d(printf("removing %d lines to smaller window\n", count));
 
     while (count > 0) {
-      if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines)) ) {
-	printf("removing line wn->line=%d\n", wn->line);
-	if ((vt->mode & VTMODE_ALTSCREEN)==0)
-	  vt_scrollback_add(vt, wn); /* add it to scrollback buffer */
-	g_free(wn);
-      }
-      /* and for 'alternate' screen */
-      if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_alt)) ) {
-	if ((vt->mode & VTMODE_ALTSCREEN)!=0)
-	  vt_scrollback_add(vt, wn); /* add it to scrollback buffer */
-	g_free(wn);
-      }
-#ifdef DOUBLE_BUFFER
-      /* repeat for backbuffer */
-      if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_back)) )
-	g_free(wn);
-#endif
+
+      /* never move the current line into the scrollback buffer, instead
+       * switch to a mode where we just dispose of the lines at the bottom
+       * of the terminal
+       */
+      if ((struct vt_line *)vt->lines.head == vt->this_line ||
+	  (struct vt_line *)vt->lines_alt.head == vt->this_line)
+	{
+	  remove_bottom = 1;
+	}
+
+      if (remove_bottom)
+	{
+	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines)) ) {
+	    g_free(wn);
+	  }
+	  
+	  /* and for 'alternate' screen */
+	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines_alt)) ) {
+	    g_free(wn);
+	  }
+	  
+	  /* repeat for backbuffer */
+	  if ( (wn = (struct vt_line *)vt_list_remtail(&vt->lines_back)) )
+	    g_free(wn);
+	}
+      else
+	{
+	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines)) ) {
+	    if ((vt->mode & VTMODE_ALTSCREEN)==0)
+	      vt_scrollback_add(vt, wn);
+	    g_free(wn);
+	  }
+	  
+	  /* and for 'alternate' screen */
+	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_alt)) ) {
+	    if ((vt->mode & VTMODE_ALTSCREEN)!=0)
+	      vt_scrollback_add(vt, wn);
+	    g_free(wn);
+	  }
+	  
+	  /* repeat for backbuffer */
+	  if ( (wn = (struct vt_line *)vt_list_remhead(&vt->lines_back)) )
+	    g_free(wn);
+	}
+
       count--;
     }
 
@@ -1625,20 +1748,18 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
 	  nn->data[j]=c;
 	}
 	g_free(wn);
+
 	vt_list_addhead(&vt->lines, (struct vt_listnode *)nn);
 	vt_list_addhead(&vt->lines_alt, (struct vt_listnode *)vt_newline(vt));
-#ifdef DOUBLE_BUFFER
 	vt_list_addhead(&vt->lines_back, (struct vt_listnode *)vt_newline(vt));
-#endif
+
 	vt->scrollbacklines--;	/* since we just nuked one */
 	vt->cursory++;		/* fix up cursor */
       } else {
 	d(printf("adding line to bottom of screen\n"));
 	/* otherwise just add blank lines to the bottom */
 	vt_list_addtail(&vt->lines, (struct vt_listnode *)vt_newline(vt));
-#ifdef DOUBLE_BUFFER
 	vt_list_addtail(&vt->lines_back, (struct vt_listnode *)vt_newline(vt));
-#endif
 	vt_list_addtail(&vt->lines_alt, (struct vt_listnode *)vt_newline(vt));
       } /* if scrollbacklines */
     }
@@ -1655,48 +1776,9 @@ void vt_resize(struct vt_em *vt, int width, int height, int pixwidth, int pixhei
   /* now, scan all lines visible, and make them the right width
    * for all 3 'buffers', onscreen, offscreen and alternate
    */
-  wn = (struct vt_line *)vt->lines.head;
-
-#ifdef DOUBLE_BUFFER
-  for (pass = 0; pass < 3; pass++) {
-#endif /* DOUBLE_BUFFER */
-
-    nn = wn->next;
-    while (nn) {
-     d(printf("wn->line=%d wn->width=%d width=%d\n", wn->line, wn->width, width));
-
-      if (wn->width != width) {
-	wn = g_realloc(wn, sizeof(struct vt_line) + (sizeof(uint32)*width));
-
-	/* re-link line into linked list */
-	wn->next->prev = wn;
-	wn->prev->next = wn;
-
-	if (wn->width > 0)
-	  c = wn->data[wn->width-1] & VTATTR_MASK;
-	else
-	  c = vt->attr;
-
-	/* if the line got bigger, fix it up */
-	for (i = wn->width; i < width; i++) {
-	  wn->data[i] = c;
-	  wn->modcount++;
-	}
-
-	wn->width = width;
-      }
-
-      wn = nn;
-      nn = nn->next;
-    }
-
-#ifdef DOUBLE_BUFFER
-    if (pass == 0)
-      wn = (struct vt_line *)vt->lines_back.head;
-    else if (pass == 1)
-      wn = (struct vt_line *)vt->lines_alt.head;
-  }
-#endif
+  vt_resize_lines((struct vt_line *) vt->lines.head, width, vt->attr);
+  vt_resize_lines((struct vt_line *) vt->lines_back.head, width, vt->attr);
+  vt_resize_lines((struct vt_line *) vt->lines_alt.head, width, vt->attr);
 
   /* re-fix 'this line' pointer */
   vt->this_line = (struct vt_line *) vt_list_index(&vt->lines, vt->cursory);
